--
-- VortexInviter.lua
-- Copyright 2009-2021 Johannes Rydh
--
-- Permission is hereby granted, free of charge, to any person obtaining
-- a copy of this software and associated documentation files (the
-- "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish,
-- distribute, sublicense, and/or sell copies of the Software, and to
-- permit persons to whom the Software is furnished to do so, subject to
-- the following conditions:
--
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
-- LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
-- OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
-- WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
--

VortexInviter = LibStub("AceAddon-3.0"):NewAddon("VortexInviter",
                                                 "AceEvent-3.0");
AceGUI = LibStub("AceGUI-3.0");

function VortexInviter:OnInitialize()
   self.frame = self:InitRaidFrame();

   SLASH_VI_INVITE1 = "/vinv";
   SLASH_VI_INVITE2 = "/vinvite";
   SlashCmdList["VI_INVITE"] = function( p ) self:Invite(p); end;
   AUTOCOMPLETE_LIST["VI_INVITE"] =
      AUTOCOMPLETE_LIST_TEMPLATES["ONLINE_NOT_IN_GROUP"];
   SLASH_VI_PREINVITE1 = "/vpinv";
   SLASH_VI_PREINVITE2 = "/vpinvite";
   SlashCmdList["VI_PREINVITE"] = function( p ) self:PreInvite(p); end;
   AUTOCOMPLETE_LIST["VI_PREINVITE"] =
      AUTOCOMPLETE_LIST_TEMPLATES["ONLINE_NOT_IN_GROUP"];
   SLASH_VI_NEW1 = "/vinew";
   SlashCmdList["VI_NEW"] = function() self:NewRaid(); end;
   SLASH_VI_SHOW1 = "/vi";
   SlashCmdList["VI_SHOW"] = function() self:Show(); end;
end

function VortexInviter:OnEnable()
   VortexInviterDB = VortexInviterDB or {};
   self.raid = VortexInviterDB;
   self:RegisterEvent( "GROUP_ROSTER_UPDATE" );
   self:RegisterEvent( "RAID_ROSTER_UPDATE" );
   self:RegisterEvent( "CHAT_MSG_WHISPER" );
end

function VortexInviter:InitRaidFrame()
   local f = CreateFrame( "Frame", "VortexInviterFrame",
                          UIParent, "SecureFrameTemplate" );

   f:SetWidth( 185 );
   f:SetHeight( 340 );
   f:EnableMouse( true );
   f:SetMovable( true );
   f:SetScript( "OnMouseDown", function() f:StartMoving(); end );
   f:SetScript( "OnMouseUp", function() f:StopMovingOrSizing(); end );
   if not f:GetLeft() then f:SetPoint( "CENTER" ); end

   local bg = f:CreateTexture( nil, "BACKGROUND" );
   bg:SetAllPoints( f );
   bg:SetColorTexture( 0, 0, 0, 0.5 );

   local s = f:CreateFontString( nil, "OVERLAY", "GameFontNormalSmall" );
   s:SetPoint( "TOP", f, "TOP", 0, -10 );
   s:SetText( "VortexInviter" );

   self.raidbtn = {};
   for i = 1,2 do
      for j = 1,20 do
         local k = (i-1)*20 + j;

         local b = CreateFrame( "Button", nil, f );
         b:SetPoint( "TOPLEFT", f, "TOPLEFT", 15+(i-1)*80, -15-j*15 );
         b:SetWidth( 75 );
         b:SetHeight( 15 );
         b.s = b:CreateFontString( nil, "OVERLAY", "GameFontHighlightSmall" );
         b.s:SetPoint( "LEFT" );
         b:RegisterForClicks( "LeftButtonDown", "RightButtonDown" );
         b:SetScript( "OnClick", function( frame, mb )
                         if mb == "LeftButton" then
                            InviteUnit( self.raid[k] );
                         elseif mb == "RightButton" then
                            self:RemovePlayer( k );
                         end
         end );
         self.raidbtn[k] = b;
      end
   end

   -- Close button
   local b = CreateFrame( "Button", nil, f, UIPanelCloseButton );
   b:SetPoint( "TOPRIGHT", f, "TOPRIGHT" );
   b:SetScript( "OnClick", function( self ) self:GetParent():Hide(); end );
   b:SetWidth( 32 );
   b:SetHeight( 32 );
   b:SetNormalTexture( "Interface/Buttons/UI-Panel-MinimizeButton-Up" );
   b:SetPushedTexture( "Interface/Buttons/UI-Panel-MinimizeButton-Down" );
   b:SetHighlightTexture(
      "Interface/Buttons/UI-Panel-MinimizeButton-Highlight" );

   f:SetScript( "OnUpdate",
                function() if f.needsUpdate then
                      self:UpdateRaidFrame(); end end );

   f.needsUpdate = true;
   f:Hide();
   return f;
end

function VortexInviter:UpdateRaidFrame()
   for i,p in ipairs( self.raid ) do
      self.raidbtn[i].s:SetText( ("|c00%s%s|r"):format(
			UnitInRaid(p) and "00FF00" or "888888", p ) );
      self.raidbtn[i]:Show();
   end
   for i = (#self.raid+1),40 do
      self.raidbtn[i]:Hide();
   end
   self.frame.needsUpdate = false;
end

function VortexInviter:Invite( player )
   self:PreInvite( player );
   InviteUnit( player );
end

function VortexInviter:PreInvite( player )
   if #self.raid < 40 then
      tinsert( self.raid, player );
   end
   self.frame.needsUpdate = true;
   self.frame:Show();
end

function VortexInviter:RemovePlayer( index )
   if( UnitInRaid( self.raid[index] ) ) then
      UninviteUnit( self.raid[index] );
   end
   tremove( self.raid, index );

   self.frame.needsUpdate = true;
   self.frame:Show();
end

function VortexInviter:GROUP_ROSTER_UPDATE()
   self.frame.needsUpdate = true;
end

function VortexInviter:RAID_ROSTER_UPDATE()
   self.frame.needsUpdate = true;
end

function VortexInviter:NewRaid()
   VortexInviterDB = {};
   self.raid = VortexInviterDB;
   self:PreInviteList();
   self.frame.needsUpdate = true;
   self.frame:Show();
end

function VortexInviter:Show()
   self.frame.needsUpdate = true;
   self.frame:Show();
end

function VortexInviter:PreInviteList()
   local inputframe = AceGUI:Create( "Frame" );
   inputframe:SetTitle( "VortexInviter" );
   inputframe:SetLayout("Fill");

   local editbox = AceGUI:Create( "MultiLineEditBox" );
   editbox:SetLabel( "Raider members to add to invite list:" );
   editbox:SetMaxLetters( 0 );
   editbox:SetFocus();
   editbox:SetCallback( "OnEnterPressed", function( widget, event, text )
      local names = { strsplit( ",", text ) };
      for i,name in ipairs(names) do
         self:PreInvite( strtrim( name ) );
      end
   end );
   inputframe:AddChild( editbox );
end

function VortexInviter:CHAT_MSG_WHISPER( _, msg, _, _, _, sender )
   if _G.VortexInviterFrame:IsShown() and ( msg == "inv" or
                                            msg == "invite" ) then
      for i,p in ipairs( self.raid ) do
         if p == sender then
            InviteUnit( sender );
            return;
         end
      end
   end
end
